# Copyright 2024 DEViantUa <t.me/deviant_ua>
# All rights reserved.


class RelictScore:
    #TOTAL  SETTING
    RGBA = "RGBA"
    
    #BACKGROUND  SETTING
    font = 25
    bacground_size = (1920,782)
    art_size = (782,782)
    splash_size = (850,782)
    blur_art = 30
    blur_splashart = 25
    opacity_art = 0.5
    opacity_splash = 0.7
    position_line = (693,-7)
    
    #UID SETTING
    font_uid = 10
    position_uid = (7,768)
    
    #NAME SETTING
    font_name_size = 25
    name_with = 262
    name_color = (255, 255, 255, 255)
    name_h_max = 53
    name_h_min = 35
    name_position = 4
    name_size = (288, 103)
    font_name_level = 17
    color_name_level = (255, 255, 255, 255)
    position_name_level = (4, 60)
    position_name_star = (2,85)
    
    #CONSTANT SETTING
    constant_size_background = (403, 63)
    constant_size= (163, 164)
    constant_size_icon = (53,53)
    constant_size_icon_opacity = 0.2
    constant_blur = 3
    constant_icon_position = (5,5)
    
    #RELICT SETTING
    relict_size = (273,134)
    relict_icon_size = (113, 113)
    relict_icon_position = (-6,21)
    relict_main_stat_icon_size = (48, 48)
    relict_main_stat_icon_position = (90,41)
    relict_position_star = (51,27)
    relict_sub_icon_size = (29, 29)
    
    #RELICT_SETS SETTING
    sets_background = (559,56)
    setst_line_size = (559,28)
    setst_name_color = (255, 200, 91, 255)
    setst_count_position = (8,4)
    setst_font = 18
    
    #STATS SETTING
    stat_font_dop = 12
    stat_font = 20
    stat_value_font = 20
    stat_line_size = (273,58)
    stat_name_size = 15
    stat_max_width = 140
    stat_icon_size = (45,45)
    stat_icon_position = (0,6)
    stat_name_position = (46,29)
    stat_x_position = 271
    stat_line_add = False

class Ticket:
    #TOTAL  SETTING
    RGBA = "RGBA"
    
    #LIGHT_CONE SETTING
    lc_bacgrount_size = (337, 448)
    lc_image_size = (298, 410)
    lc_image_position = (19, 23)
    
    lc_frame_position = (32, 29)
    
    #BACKGROUND  SETTING
    font = 25
    bacground_default_color = (30,30,30,255)
    bacground_size = (1924,802)
    background_splash_size = (1341,802)
    art_size = (693,802)
    splash_size = (582,802)
    blur_art = None
    blur_splashart = None
    opacity_art = None
    opacity_splash = None
    position_shadow = (1341,0)
    position_line = (0,0)
    position_art = (1341,0)
    position_splash_art = (1342,0)
    
    #UID SETTING
    font_uid = 10
    position_uid = (1443 ,779)
    
    #NAME SETTING
    font_name_size = 25
    name_with = 262
    name_color = (255, 255, 255, 255)
    name_h_max = 53
    name_h_min = 35
    name_position = 4
    name_size = (288, 103)
    font_name_level = 17
    color_name_level = (255, 255, 255, 255)
    position_name_level = (4, 60)
    position_name_star = (2,85)
    
    #CONSTANT SETTING
    constant_size_background = (403, 63)
    constant_size= (163, 164)
    constant_size_icon = (53,53)
    constant_size_icon_opacity = 0.2
    constant_blur = 3
    constant_icon_position = (5,5)
    
    #RELICT SETTING
    relict_size = (273,134)
    relict_icon_size = (113, 113)
    relict_icon_position = (-6,21)
    relict_main_stat_icon_size = (38, 38)
    relict_main_stat_icon_position = (77,0)
    relict_position_star = (0,20)
    relict_sub_icon_size = (38, 38)
    
    #RELICT_SETS SETTING
    sets_background = (807,545)
    setst_line_size = (394,38)
    setst_name_color = (255, 185, 85, 255)
    setst_count_position = (7,2)
    setst_font = 15
    
    #STATS SETTING
    stat_font_dop = 18
    stat_font = 20
    stat_value_font = 30
    stat_line_size = (434,52)
    stat_name_size = 20
    stat_max_width = 275
    
    stat_y_no_dop = 15
    stat_y_yes_dop = 3
    stat_y_dop = 32
    
    stat_icon_size = (52,52)
    stat_icon_position = (0,0)
    stat_name_position = (59,27)
    stat_x_position = 432
    stat_line_add = False